	  DOSSEG
	  .MODEL  small

_CODE  SEGMENT WORD PUBLIC 'CODE'	 ;Code segement
_CODE  ENDS
_DATA  SEGMENT WORD PUBLIC 'DATA'	 ;Data segement
_DATA  ENDS
STACK SEGMENT WORD PUBLIC 'STACK'	 ;Stack segement
STACK ENDS


DGROUP GROUP _CODE, _DATA, STACK

	   ASSUME CS:DGROUP,DS:DGROUP,SS:DGROUP,ES:DGROUP

CR	EQU    0DH
LF	EQU    0AH


_DATA		SEGMENT PUBLIC
		PUBLIC CURSOR_POSITION, CHAR_COUNT, ATTRIBUTE
CURSOR_POSITION  DW 0			;Defines the row/column of cursor
CHAR_COUNT	 DW 0			;Defines the count of characters of highlight block.
ATTRIBUTE	 DB 0			;Defines the type of attribute for highlight block.

_DATA		ENDS



_CODE		SEGMENT PUBLIC

;
;WRITE ATTRIBUTE N TIMES
; PROCEDURE 
;
	PUBLIC	WRITE_ATTRIBUTE_N_TIMES
;-----------------------------------------------------------------------;
; This procedure sets the attribute for N characters starting at the	;
; current cursor position.						;
;									;
;	CX	Number of characters to set attribute for		;
;	DL	New attribute for characters				;
;									;
; Uses:		CURSOR_RIGHT						;
;-----------------------------------------------------------------------;
WRITE_ATTRIBUTE_N_TIMES   PROC	FAR

	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	MOV	BL,DL			;Set attribute to new attribute
	XOR	BH,BH			;Set display page to zero
	MOV	DX,CX			;CX is used by the BIOS routines
	MOV	CX,1			;Set attribute for one character
ATTR_LOOP:
	MOV	AH,8			;Read character under cursor
	INT	10H
	MOV	AH,9			;Write attribute/character
	INT	10H
	CALL	FAR PTR CURSOR_RIGHT
	DEC	DX			;Set attribute for N characters?
	JNZ	ATTR_LOOP		;No, continue
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RETF

WRITE_ATTRIBUTE_N_TIMES 	ENDP

;
; SEND CR AND LF 
; PROCEDURE 
;
	PUBLIC	SEND_CRLF
;-----------------------------------------------------------------------;
; This procedure just sends a carriage return, line feed pair to the	;
; display, using the DOS routines so that scrolling will be handled	;
; correctly.								;
;-----------------------------------------------------------------------;
SEND_CRLF	PROC	NEAR

	PUSH	AX
	PUSH	DX
	MOV	AH,2			;Ask for character output function
	MOV	DL,CR			;Send a carriage-return character
	INT	21h
	MOV	DL,LF			;Send a line-feed character
	INT	21h
	POP	DX
	POP	AX
	RET

SEND_CRLF	ENDP

;
; CUROSR RIGHT 
; PROCEDURE 
;
	PUBLIC	CURSOR_RIGHT
;-----------------------------------------------------------------------;
; This procedure moves the cursor one position to the right, or to the	;
; next line if it was at the end of a line.				;
;									;
; Uses:		SEND_CRLF						;
;-----------------------------------------------------------------------;
CURSOR_RIGHT	PROC	FAR

	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	MOV	AH,3		;Read the current cursor position
	MOV	BH,0		;On page zero
	INT	10h		;Read cursor position
	MOV	AH,2		;Set new cursor position
	INC	DL		;Set column to next position
	CMP	DL,79		;Did we pass column 79?
	JBE	MOVE_CURSOR_NOW ;No, then move the cursor
	CALL	SEND_CRLF	;Yes, go to next line
	JMP	DONE_CURSOR_RIGHT
MOVE_CURSOR_NOW:
	INT	10h		;Move the cursor
DONE_CURSOR_RIGHT:
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RETF

CURSOR_RIGHT	ENDP

;
; CLEAR SCREEN 
; PROCEDURE 
;
	PUBLIC	CLEAR_SCREEN
;-----------------------------------------------------------------------;
;  This procedure clears the entire screen.				;
;-----------------------------------------------------------------------;
CLEAR_SCREEN	PROC	FAR

	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	XOR	AL,AL			;Blank entire window
	XOR	CX,CX			;Set upper left to (0,0)
	MOV	DH,24			;Set lower right to (24,79)
	MOV	DL,79
	MOV	BH,7			;Use normal attribute for blanks
	MOV	AH,6			;Call for scroll function
	INT	10h			;Let the ROM BIOS do the work
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RETF

CLEAR_SCREEN	ENDP

;
; WRITE CHARACTER 
; PROCEDURE 
;
	PUBLIC	WRITE_CHAR
;-----------------------------------------------------------------------;
; This procedure outputs a character to the screen using the ROM BIOS	;
; routines so that characters such as backspace, etc., are treated as	;
; any other character and displayed.					;
;   This procedure must do a bit of work to update the cursor position.	;
;									;
;	DL	byte to print on screen.				;
;									;
; Uses:		CURSOR_RIGHT						;
;-----------------------------------------------------------------------;
WRITE_CHAR	PROC	FAR

	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	MOV	AH,8		;Call for input character/attribute
	MOV	BH,0		;Set to display page 0
	INT	10H		;Read character and attribute
	MOV	BL,AH		;Set attribute that was read at cursor position
	MOV	AH,9		;Call for output of character/attribute
	MOV	BH,0		;Set to display page 0
	MOV	CX,1		;Write only one character
	MOV	AL,DL		;Character to write
	INT	10H		;Write character and attribute
	CALL	FAR PTR CURSOR_RIGHT ;Now move the cursor to the next position
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RETF

WRITE_CHAR	ENDP

;
; MOVE CURSOR 
; PROCEDURE 
;
	PUBLIC	MOVE_CURSOR
;-----------------------------------------------------------------------;
; This procedure moves the cursor to position on the screen indicated	;
; in DX.  The BIOS is used.						;
;									;
;	DH	Row (X) 						;
;	DL	Column (x)						;
;									;
;-----------------------------------------------------------------------;
MOVE_CURSOR	PROC   FAR

	PUSH	AX			;On entry DH has Row (Y),DL has Column (X)
	PUSH	BX
	MOV	BH,0			;Display page 0
	MOV	AH,2			;Call for SET CURSOR POSITION
	INT	10h			;Let the ROM BIOS do the work
	POP	BX
	POP	AX
	RETF

MOVE_CURSOR	ENDP

;
; READ CURSOR 
; PROCEDURE 
;
	PUBLIC	READ_CURSOR_POSITION
;-----------------------------------------------------------------------;
; This procedure doesn't follow the conventions for returning		;
; information in the AX register so that it can be used easily with	;
; MOVE_CURSOR								;
;									;
; Returns:	DH,DL	Row, column of cursor				;
;-----------------------------------------------------------------------;
READ_CURSOR_POSITION	PROC	FAR
	PUSH	AX
	PUSH	BX
	PUSH	CX
	MOV	AH,3			;Ask for current cursor position
	MOV	BH,0			;On page 0
	INT	10h			;Return information in DX
	POP	CX
	POP	BX
	POP	AX
	RETF
READ_CURSOR_POSITION	ENDP

;
; HIGHLIGHT BLOCK 
; PROCEDURE 
;
	PUBLIC	HIGHLIGHT_BLOCK
;-----------------------------------------------------------------------;
; This procedure changes the attribute of a section of the screen.	;
; Only the attribute is changed.					;
;									;
; CURSOR_POSITION   row/column to move cursor				;
; CHAR_COUNT	     amount of characters for highlight block		;
; ATTRIBUTE	    Type of attribute for highlight block		;
;									;
; Uses:   WRTIE_ATTRIBUTES_N_TIMES, MOVE_CURSOR, CURSOR_RIGHT		;
;-----------------------------------------------------------------------;
HIGHLIGHT_BLOCK  PROC	 FAR

	  MOV DX,[CURSOR_POSITION]	 ;DX = new row/column for cursor.
	  CALL MOVE_CURSOR		 ;Move cursor to new position.
	  MOV CX,[CHAR_COUNT]		 ;Set CX for how many characters to highlight.
	  MOV DL,[ATTRIBUTE]		 ;Set DL for normal highlight code.
	  CALL WRITE_ATTRIBUTE_N_TIMES	 ;Write attribute at cursor position.
	  RETF

HIGHLIGHT_BLOCK  ENDP


;
;  CURSOR OFF 
; PROCEDURE 
;
	PUBLIC	CURSOR_OFF
;-----------------------------------------------------------------------;
; This procedure changes turns the cursor off				;
;-----------------------------------------------------------------------;
CURSOR_OFF	 PROC	 FAR

	  MOV AH,1			 ;Set cursor type.
	  MOV CX,2000H
	  INT 10H			 ;BIOS video call.
	  RETF

CURSOR_OFF	 ENDP

;
;  CURSOR ON  
; PROCEDURE 
;
	PUBLIC	CURSOR_ON
;-----------------------------------------------------------------------;
; This procedure changes turns the cursor on				;
;-----------------------------------------------------------------------;
CURSOR_ON	 PROC	 FAR

	  MOV AH,1			 ;Set cursor type.
	  MOV CX,0607H
	  INT 10H			 ;BIOS video call.
	  RETF

CURSOR_ON	 ENDP

_CODE		ENDS

		END
